@@ -1398,13 +1398,10 @@ bool ClassLoaderDataGraph::do_unloading(bool clean_previous_versions) {
   // Klassesoto delete.
 
   // FIXME: dcevm - block asserts in MetadataOnStackMark
-  bool walk_all_metadata = false;
-  if (!AllowEnhancedClassRedefinition) {
-    walk_all_metadata = clean_previous_versions &&
+  bool walk_all_metadata = clean_previous_versions &&
                              JvmtiExport::has_redefined_a_class() &&
                              InstanceKlass::has_previous_versions_and_reset();
-    MetadataOnStackMark md_on_stack(walk_all_metadata);
-  }
+  MetadataOnStackMark md_on_stack(walk_all_metadata, Universe::is_redefining_gc_run());
 
   // Save previous _unloading pointer for CMS which may add to unloading list before
   // purging and we don't want to rewalk the previously unloaded class loader data.
