@@ -52,8 +52,6 @@ class Dictionary : public Hashtable<InstanceKlass*, mtClass> {
 
   DictionaryEntry* get_entry(int index, unsigned int hash, Symbol* name);
 
-  void clean_cached_protection_domains(DictionaryEntry* probe);
-
 protected:
   static size_t entry_size();
 public:
@@ -81,6 +79,8 @@ public:
   void unlink();
   void remove_classes_in_error_state();
 
+  void clean_cached_protection_domains();
+
   // Protection domains
   InstanceKlass* find(unsigned int hash, Symbol* name, Handle protection_domain);
   bool is_valid_protection_domain(unsigned int hash,
@@ -119,6 +119,7 @@ public:
 
   void rollback_redefinition();
 
+  // (DCEVM) return old class if redefining in AllowEnhancedClassRedefinition, otherwise return "k"
   static InstanceKlass* old_if_redefined(InstanceKlass* k) {
     return (k != NULL && k->is_redefining()) ? ((InstanceKlass* )k->old_version()) : k;
   }
