@@ -144,6 +144,7 @@ class G1CollectedHeap : public CollectedHeap {
   friend class G1ParTask;
   friend class G1PLABAllocator;
   friend class G1PrepareCompactClosure;
+  friend class G1FullGCPrepareTask;
 
   // Other related classes.
   friend class HeapRegionClaimer;
@@ -697,7 +698,7 @@ public:
   // which had been allocated by alloc_archive_regions. This should be called
   // rather than fill_archive_regions at JVM init time if the archive file
   // mapping failed, with the same non-overlapping and sorted MemRegion array.
-  void dealloc_archive_regions(MemRegion* range, size_t count);
+  void dealloc_archive_regions(MemRegion* range, size_t count, bool is_open);
 
   oop materialize_archived_object(oop obj);
 
@@ -1116,6 +1117,8 @@ public:
 
   // Iteration functions.
 
+  void object_par_iterate(ObjectClosure* cl);
+
   // Iterate over all objects, calling "cl.do_object" on each.
   virtual void object_iterate(ObjectClosure* cl);
 
