@@ -36,7 +36,6 @@
 #include "memory/metaspaceShared.hpp"
 #include "memory/resourceArea.hpp"
 #include "memory/iterator.inline.hpp"
-#include "gc/serial/markSweep.hpp"
 #include "oops/fieldStreams.hpp"
 #include "oops/klassVtable.hpp"
 #include "oops/oop.inline.hpp"
@@ -55,6 +54,8 @@
 #include "utilities/events.hpp"
 #include "oops/constantPool.inline.hpp"
 #include "gc/cms/cmsHeap.hpp"
+#include "gc/g1/g1CollectedHeap.hpp"
+#include "gc/shared/dcevmSharedGC.hpp"
 
 Array<Method*>* VM_EnhancedRedefineClasses::_old_methods = NULL;
 Array<Method*>* VM_EnhancedRedefineClasses::_new_methods = NULL;
@@ -67,21 +68,19 @@ int         VM_EnhancedRedefineClasses::_deleted_methods_length  = 0;
 int         VM_EnhancedRedefineClasses::_added_methods_length    = 0;
 Klass*      VM_EnhancedRedefineClasses::_the_class_oop = NULL;
 
-/**
- * Create new instance of enhanced class redefiner.
- *
- * This class implements VM_GC_Operation - the usual usage should be:
- *     VM_EnhancedRedefineClasses op(class_count, class_definitions, jvmti_class_load_kind_redefine);
- *     VMThread::execute(&op);
- * Which
- *
- * @param class_count size of class_defs
- * @param class_defs class definition - either new class or redefined class
- *               note that this is not the final array of classes to be redefined
- *               we need to scan for all affected classes (e.g. subclasses) and
- *               caculcate redefinition for them as well.
- * @param class_load_kind always jvmti_class_load_kind_redefine
- */
+//
+// Create new instance of enhanced class redefiner.
+//
+// This class implements VM_GC_Operation - the usual usage should be:
+//     VM_EnhancedRedefineClasses op(class_count, class_definitions, jvmti_class_load_kind_redefine);
+//     VMThread::execute(&op);
+// Which
+//  - class_count size of class_defs
+//  - class_defs class definition - either new class or redefined class
+//               note that this is not the final array of classes to be redefined
+//               we need to scan for all affected classes (e.g. subclasses) and
+//               calculate redefinition for them as well.
+// @param class_load_kind always jvmti_class_load_kind_redefine
 VM_EnhancedRedefineClasses::VM_EnhancedRedefineClasses(jint class_count, const jvmtiClassDefinition *class_defs, JvmtiClassLoadKind class_load_kind) :
         VM_GC_Operation(Universe::heap()->total_collections(), GCCause::_heap_inspection, Universe::heap()->total_full_collections(), true) {
   _affected_klasses = NULL;
@@ -97,12 +96,10 @@ static inline InstanceKlass* get_ik(jclass def) {
   return InstanceKlass::cast(java_lang_Class::as_Klass(mirror));
 }
 
-/**
- * Start the redefinition:
- * - Load new class definitions - @see load_new_class_versions
- * - Start mark&sweep GC.
- * @return true if success, otherwise all chnages are rollbacked.
- */
+// Start the redefinition:
+// - Load new class definitions - @see load_new_class_versions
+// - Start mark&sweep GC.
+// - true if success, otherwise all chnages are rollbacked.
 bool VM_EnhancedRedefineClasses::doit_prologue() {
 
   if (_class_count == 0) {
@@ -175,9 +172,7 @@ bool VM_EnhancedRedefineClasses::doit_prologue() {
   return true;
 }
 
-/**
- * Closer for static fields - copy value from old class to the new class.
- */
+// Closer for static fields - copy value from old class to the new class.
 class FieldCopier : public FieldClosure {
   public:
   void do_field(fieldDescriptor* fd) {
@@ -222,6 +217,20 @@ void VM_EnhancedRedefineClasses::mark_as_scavengable(nmethod* nm) {
   }
 }
 
+void VM_EnhancedRedefineClasses::unregister_nmethod_g1(nmethod* nm) {
+  // It should work not only for G1 but also for another GCs, but this way is safer now
+  if (!nm->is_zombie() && !nm->is_unloaded()) {
+    Universe::heap()->unregister_nmethod(nm);
+  }
+}
+
+void VM_EnhancedRedefineClasses::register_nmethod_g1(nmethod* nm) {
+  // It should work not only for G1 but also for another GCs, but this way is safer now
+  if (!nm->is_zombie() && !nm->is_unloaded()) {
+    Universe::heap()->register_nmethod(nm);
+  }
+}
+
 // TODO comment
 struct StoreBarrier {
   // TODO: j10 review change ::oop_store -> HeapAccess<>::oop_store
@@ -236,9 +245,7 @@ struct StoreNoBarrier {
   template <class T> static void oop_store(T* p) { RawAccess<>::oop_store(p, oop(NULL)); }
 };
 
-/**
-  Closure to scan all heap objects and update method handles
-*/
+// Closure to scan all heap objects and update method handles
 template <class S>
 class ChangePointersOopClosure : public BasicOopIterateClosure {
   // import java_lang_invoke_MemberName.*
@@ -247,7 +254,6 @@ class ChangePointersOopClosure : public BasicOopIterateClosure {
     REFERENCE_KIND_MASK  = java_lang_invoke_MemberName::MN_REFERENCE_KIND_MASK,
   };
 
-
   bool update_member_name(oop obj) {
     int flags    =       java_lang_invoke_MemberName::flags(obj);
     int ref_kind =       (flags >> REFERENCE_KIND_SHIFT) & REFERENCE_KIND_MASK;
@@ -262,7 +268,7 @@ class ChangePointersOopClosure : public BasicOopIterateClosure {
           // Note: we might set NULL at this point, which should force AbstractMethodError at runtime
           Thread *thread = Thread::current();
           CallInfo info(new_method, newest, thread);
-          Handle objHandle(thread, obj);  // TODO : review thread
+          Handle objHandle(thread, obj);
           MethodHandles::init_method_MemberName(objHandle, info);
         } else {
           java_lang_invoke_MemberName::set_method(obj, NULL);
@@ -289,7 +295,7 @@ class ChangePointersOopClosure : public BasicOopIterateClosure {
           InstanceKlass* ik_new = InstanceKlass::cast(k->newest_version());
           fieldDescriptor fd_new;
           if (ik_new->find_local_field(fd.name(), fd.signature(), &fd_new)) {
-            Handle objHandle(Thread::current(), obj);  // TODO : review thread
+            Handle objHandle(Thread::current(), obj);
             MethodHandles::init_field_MemberName(objHandle, fd_new, MethodHandles::ref_kind_is_setter(ref_kind));
           } else {
             // Matching field is not found in new version, not much we can do here.
@@ -382,14 +388,12 @@ class ChangePointersOopClosure : public BasicOopIterateClosure {
   }
 };
 
-/**
- * Closure to scan all objects on heap for objects of changed classes
- *   - if the fields are compatible, only update class definition reference
- *   - otherwise if the new object size is smaller then old size, reshufle
- *          the fields and fill the gap with "dead_space"
- *   - otherwise set the _needs_instance_update flag, we need to do full GC
- *          and reshuffle object positions durring mark&sweep
- */
+// Closure to scan all objects on heap for objects of changed classes
+//  - if the fields are compatible, only update class definition reference
+//  - otherwise if the new object size is smaller then old size, reshufle
+//         the fields and fill the gap with "dead_space"
+//  - otherwise set the _needs_instance_update flag, we need to do full GC
+//         and reshuffle object positions durring mark&sweep
 class ChangePointersObjectClosure : public ObjectClosure {
   private:
 
@@ -441,7 +445,7 @@ public:
           src->set_klass(obj->klass()->new_version());
           //  FIXME: instance updates...
           //guarantee(false, "instance updates!");
-          MarkSweep::update_fields(obj, src, new_klass->update_information());
+          DcevmSharedGC::update_fields(obj, src, new_klass->update_information());
         }
       } else {
         obj->set_klass(obj->klass()->new_version());
@@ -451,22 +455,22 @@ public:
 };
 
 
-/**
-  Main transformation method - runs in VM thread.
-
-  - UseSharedSpaces - TODO what does it mean?
-  - for each sratch class call redefine_single_class
-  - clear code cache (flush_dependent_code)
-  - iterate the heap and update object defintions, check it old/new class fields
-       are compatible. If new class size is smaller then old, it can be solved directly here.
-  - iterate the heap and update method handles to new version
-  - Swap marks to have same hashcodes
-  - copy static fields
-  - notify JVM of the modification
-*/
+// Main transformation method - runs in VM thread.
+//  - for each scratch class call redefine_single_class
+//  - clear code cache (flush_dependent_code)
+//  - iterate the heap and update object definitions, check it old/new class fields
+//       are compatible. If new class size is smaller then old, it can be solved directly here.
+//  - iterate the heap and update method handles to new version
+//  - Swap marks to have same hashcodes
+//  - copy static fields
+//  - notify JVM of the modification
 void VM_EnhancedRedefineClasses::doit() {
   Thread *thread = Thread::current();
 
+  if (log_is_enabled(Info, redefine, class, timer)) {
+    _timer_vm_op_doit.start();
+  }
+
 #if INCLUDE_CDS
   if (UseSharedSpaces) {
     // Sharing is enabled so we remap the shared readonly space to
@@ -502,9 +506,6 @@ void VM_EnhancedRedefineClasses::doit() {
     ClearCpoolCacheAndUnpatch clear_cpool_cache(thread);
     ClassLoaderDataGraph::classes_do(&clear_cpool_cache);
 
-
-    // SystemDictionary::methods_do(fix_invoke_method);
-
   // JSR-292 support
   if (_any_class_has_resolved_methods) {
     bool trace_name_printed = false;
@@ -522,12 +523,37 @@ void VM_EnhancedRedefineClasses::doit() {
     // mark such nmethod's as "scavengable".
     // For now, mark all nmethod's as scavengable that are not scavengable already
     if (ScavengeRootsInCode) {
-      CodeCache::nmethods_do(mark_as_scavengable);
+      if (UseG1GC) {
+        // G1 holds references to nmethods in regions based on oops values. Since oops in nmethod can be changed in ChangePointers* closures
+        // we unregister nmethods from G1 heap, then closures are processed (oops are changed) and finally we register nmethod to G1 again
+        CodeCache::nmethods_do(unregister_nmethod_g1);
+      } else {
+        CodeCache::nmethods_do(mark_as_scavengable);
+      }
     }
 
     Universe::heap()->ensure_parsability(false);
-    Universe::heap()->object_iterate(&objectClosure);
+    if (UseG1GC) {
+      if (log_is_enabled(Info, redefine, class, timer)) {
+        _timer_heap_iterate.start();
+      }
+      G1CollectedHeap::heap()->object_par_iterate(&objectClosure);
+      _timer_heap_iterate.stop();
+    } else {
+      if (log_is_enabled(Info, redefine, class, timer)) {
+        _timer_heap_iterate.start();
+      }
+      Universe::heap()->object_iterate(&objectClosure);
+      _timer_heap_iterate.stop();
+    }
+
     Universe::root_oops_do(&oopClosureNoBarrier);
+
+    if (UseG1GC) {
+      // this should work also for other GCs
+      CodeCache::nmethods_do(register_nmethod_g1);
+    }
+
   }
   log_trace(redefine, class, obsolete, metadata)("After updating instances");
 
@@ -580,11 +606,19 @@ void VM_EnhancedRedefineClasses::doit() {
 
   if (objectClosure.needs_instance_update()) {
     // Do a full garbage collection to update the instance sizes accordingly
+
+    if (log_is_enabled(Info, redefine, class, timer)) {
+      _timer_heap_full_gc.start();
+    }
+
     Universe::set_redefining_gc_run(true);
     notify_gc_begin(true);
+    // TODO: check _metadata_GC_clear_soft_refs with ScavengeRootsInCode
     Universe::heap()->collect_as_vm_thread(GCCause::_heap_inspection);
     notify_gc_end();
     Universe::set_redefining_gc_run(false);
+
+    _timer_heap_full_gc.stop();
   }
 
   // Unmark Klass*s as "redefining"
@@ -632,13 +666,12 @@ void VM_EnhancedRedefineClasses::doit() {
   }
 #endif
 
+  _timer_vm_op_doit.stop();
 }
 
-/**
- * Cleanup - runs in JVM thread
- *  - free used memory
- *  - end GC
- */
+// Cleanup - runs in JVM thread
+//  - free used memory
+//  - end GC
 void VM_EnhancedRedefineClasses::doit_epilogue() {
   VM_GC_Operation::doit_epilogue();
 
@@ -657,24 +690,20 @@ void VM_EnhancedRedefineClasses::doit_epilogue() {
   if (log_is_enabled(Info, redefine, class, timer)) {
     // Used to have separate timers for "doit" and "all", but the timer
     // overhead skewed the measurements.
-    jlong doit_time = _timer_rsc_phase1.milliseconds() +
-                      _timer_rsc_phase2.milliseconds();
-    jlong all_time = _timer_vm_op_prologue.milliseconds() + doit_time;
+    jlong all_time = _timer_vm_op_prologue.milliseconds() + _timer_vm_op_doit.milliseconds();
 
     log_info(redefine, class, timer)
       ("vm_op: all=" JLONG_FORMAT "  prologue=" JLONG_FORMAT "  doit=" JLONG_FORMAT,
-       all_time, _timer_vm_op_prologue.milliseconds(), doit_time);
+       all_time, _timer_vm_op_prologue.milliseconds(), _timer_vm_op_doit.milliseconds());
     log_info(redefine, class, timer)
-      ("redefine_single_class: phase1=" JLONG_FORMAT "  phase2=" JLONG_FORMAT,
-       _timer_rsc_phase1.milliseconds(), _timer_rsc_phase2.milliseconds());
+      ("doit: heap iterate=" JLONG_FORMAT "  fullgc=" JLONG_FORMAT,
+       _timer_heap_iterate.milliseconds(), _timer_heap_full_gc.milliseconds());
   }
 }
 
-/**
- * Exclude java primitives and arrays from redefinition
- * @param klass_mirror  pointer to the klass
- * @return true if is modifiable
- */
+// Exclude java primitives and arrays from redefinition
+//  - klass_mirror  pointer to the klass
+//  - true if is modifiable
 bool VM_EnhancedRedefineClasses::is_modifiable_class(oop klass_mirror) {
   // classes for primitives cannot be redefined
   if (java_lang_Class::is_primitive(klass_mirror)) {
@@ -693,17 +722,12 @@ bool VM_EnhancedRedefineClasses::is_modifiable_class(oop klass_mirror) {
   return true;
 }
 
-/**
-  Load and link new classes (either redefined or affected by redefinition - subclass, ...)
-
-  - find sorted affected classes
-  - resolve new class
-  - calculate redefine flags (field change, method change, supertype change, ...)
-  - calculate modified fields and mapping to old fields
-  - link new classes
-
-  The result is sotred in _affected_klasses(old definitions) and _new_classes(new definitions) arrays.
-*/
+// Load and link new classes (either redefined or affected by redefinition - subclass, ...)
+//  - find sorted affected classes
+//  - resolve new class
+//  - calculate redefine flags (field change, method change, supertype change, ...)
+//  - calculate modified fields and mapping to old fields
+//  - link new classes
 jvmtiError VM_EnhancedRedefineClasses::load_new_class_versions(TRAPS) {
 
   _affected_klasses = new (ResourceObj::C_HEAP, mtInternal) GrowableArray<Klass*>(_class_count, true);
@@ -898,12 +922,10 @@ jvmtiError VM_EnhancedRedefineClasses::load_new_class_versions(TRAPS) {
   return JVMTI_ERROR_NONE;
 }
 
-/**
-  Calculated the difference between new and old class  (field change, method change, supertype change, ...).
-*/
+ // Calculated the difference between new and old class  (field change, method change, supertype change, ...).
 int VM_EnhancedRedefineClasses::calculate_redefinition_flags(InstanceKlass* new_class) {
   int result = Klass::NoRedefinition;
-  log_info(redefine, class, load)("Comparing different class versions of class %s",new_class->name()->as_C_string());
+  log_debug(redefine, class, load)("Comparing different class versions of class %s",new_class->name()->as_C_string());
 
   assert(new_class->old_version() != NULL, "must have old version");
   InstanceKlass* the_class = InstanceKlass::cast(new_class->old_version());
@@ -1183,14 +1205,11 @@ int VM_EnhancedRedefineClasses::calculate_redefinition_flags(InstanceKlass* new_
 }
 
 
-/**
-  Searches for the class bytecode of the given class and returns it as a byte array.
-
-  @param the_class definition of a class, either existing class or new_class
-  @param class_bytes - if the class is redefined, it contains new class definition, otherwise just original class bytecode.
-  @param class_byte_count - size of class_bytes
-  @param not_changed - new_class not available or same as current class
-*/
+// Searches for the class bytecode of the given class and returns it as a byte array.
+//  - the_class definition of a class, either existing class or new_class
+//  - class_bytes - if the class is redefined, it contains new class definition, otherwise just original class bytecode.
+//  - class_byte_count - size of class_bytes
+//  - not_changed - new_class not available or same as current class
 jvmtiError VM_EnhancedRedefineClasses::find_class_bytes(InstanceKlass* the_class, const unsigned char **class_bytes, jint *class_byte_count, jboolean *not_changed) {
 
   *not_changed = false;
@@ -1233,11 +1252,9 @@ jvmtiError VM_EnhancedRedefineClasses::find_class_bytes(InstanceKlass* the_class
   return JVMTI_ERROR_NONE;
 }
 
-/**
-  Calculate difference between non static fields of old and new class and store the info into new class:
-     instanceKlass->store_update_information
-     instanceKlass->copy_backwards
-*/
+// Calculate difference between non static fields of old and new class and store the info into new class:
+//     instanceKlass->store_update_information
+//     instanceKlass->copy_backwards
 void VM_EnhancedRedefineClasses::calculate_instance_update_information(Klass* new_version) {
 
   class CalculateFieldUpdates : public FieldClosure {
@@ -1348,9 +1365,7 @@ void VM_EnhancedRedefineClasses::calculate_instance_update_information(Klass* ne
   }
 }
 
-/**
-  Rollback all changes - clear new classes from the system dictionary, return old classes to directory, free memory.
-*/
+// Rollback all changes - clear new classes from the system dictionary, return old classes to directory, free memory.
 void VM_EnhancedRedefineClasses::rollback() {
   log_info(redefine, class, load)("Rolling back redefinition, result=%d", _res);
   ClassLoaderDataGraph::rollback_redefinition();
@@ -1389,14 +1404,16 @@ void VM_EnhancedRedefineClasses::unpatch_bytecode(Method* method) {
 
     if (code == Bytecodes::_breakpoint) {
       int bci = method->bci_from(bcp);
-      code = method->orig_bytecode_at(bci);
-      java_code = Bytecodes::java_code(code);
-      if (code != java_code &&
-           (java_code == Bytecodes::_getfield ||
-            java_code == Bytecodes::_putfield ||
-            java_code == Bytecodes::_aload_0)) {
-        // Let breakpoint table handling unpatch bytecode
-        method->set_orig_bytecode_at(bci, java_code);
+      code = method->orig_bytecode_at(bci, true);
+      if (code != Bytecodes::_shouldnotreachhere) {
+        java_code = Bytecodes::java_code(code);
+        if (code != java_code &&
+             (java_code == Bytecodes::_getfield ||
+              java_code == Bytecodes::_putfield ||
+              java_code == Bytecodes::_aload_0)) {
+          // Let breakpoint table handling unpatch bytecode
+          method->set_orig_bytecode_at(bci, java_code);
+        }
       }
     } else {
       java_code = Bytecodes::java_code(code);
@@ -1429,7 +1446,7 @@ void VM_EnhancedRedefineClasses::unpatch_bytecode(Method* method) {
 // arrayKlassOops. See Open Issues in jvmtiRedefineClasses.hpp.
 void VM_EnhancedRedefineClasses::ClearCpoolCacheAndUnpatch::do_klass(Klass* k) {
   if (!k->is_instance_klass()) {
-	return;
+    return;
   }
 
   HandleMark hm(_thread);
@@ -1465,15 +1482,17 @@ void VM_EnhancedRedefineClasses::ClearCpoolCacheAndUnpatch::do_klass(Klass* k) {
     }
   }
 
-  // DCEVM - clear whole cache (instead special methods for class/method update in standard redefinition)
-  ConstantPoolCache* cp_cache = other_cp->cache();
-  if (cp_cache != NULL) {
-    cp_cache->clear_entries();
-  }
+  if (!k->is_deoptimization_excl()) {
+    // DCEVM - clear whole cache (instead special methods for class/method update in standard redefinition)
+    ConstantPoolCache* cp_cache = other_cp->cache();
+    if (cp_cache != NULL) {
+      cp_cache->clear_entries();
+    }
 
-  // If bytecode rewriting is enabled, we also need to unpatch bytecode to force resolution of zeroed entries
-  if (RewriteBytecodes) {
-    ik->methods_do(unpatch_bytecode);
+    // If bytecode rewriting is enabled, we also need to unpatch bytecode to force resolution of zeroed entries
+    if (RewriteBytecodes) {
+      ik->methods_do(unpatch_bytecode);
+    }
   }
 }
 
@@ -1493,30 +1512,6 @@ void VM_EnhancedRedefineClasses::MethodDataCleaner::do_klass(Klass* k) {
   }
 }
 
-void VM_EnhancedRedefineClasses::fix_invoke_method(Method* method) {
-
-  constantPoolHandle other_cp = constantPoolHandle(method->constants());
-
-  for (int i = 0; i < other_cp->length(); i++) {
-    if (other_cp->tag_at(i).is_klass()) {
-      Klass* klass = other_cp->resolved_klass_at(i);
-      if (klass->new_version() != NULL) {
-        // Constant pool entry points to redefined class -- update to the new version
-        other_cp->klass_at_put(i, klass->newest_version());
-      }
-      assert(other_cp->resolved_klass_at(i)->new_version() == NULL, "Must be new klass!");
-    }
-  }
-
-  ConstantPoolCache* cp_cache = other_cp->cache();
-  if (cp_cache != NULL) {
-    cp_cache->clear_entries();
-  }
-
-}
-
-
-
 void VM_EnhancedRedefineClasses::update_jmethod_ids() {
   for (int j = 0; j < _matching_methods_length; ++j) {
     Method* old_method = _matching_old_methods[j];
@@ -1545,9 +1540,7 @@ void VM_EnhancedRedefineClasses::update_jmethod_ids() {
   }
 }
 
-/**
-  Set method as obsolete / old / deleted.
-*/
+// Set method as obsolete / old / deleted.
 void VM_EnhancedRedefineClasses::check_methods_and_mark_as_obsolete() {
   for (int j = 0; j < _matching_methods_length; ++j/*, ++old_index*/) {
     Method* old_method = _matching_old_methods[j];
@@ -1617,7 +1610,7 @@ class TransferNativeFunctionRegistration {
 
   // Recursively search the binary tree of possibly prefixed method names.
   // Iteration could be used if all agents were well behaved. Full tree walk is
-  // more resilent to agents not cleaning up intermediate methods.
+  // more resilient to agents not cleaning up intermediate methods.
   // Branch at each depth in the binary tree is:
   //    (1) without the prefix.
   //    (2) with the prefix.
@@ -1722,7 +1715,7 @@ void VM_EnhancedRedefineClasses::transfer_old_native_function_registrations(Inst
   transfer.transfer_registrations(_matching_old_methods, _matching_methods_length);
 }
 
-// DCEVM - it always deoptimases everything! (because it is very difficult to find only correct dependencies)
+// DCEVM - it always deoptimizes everything! (because it is very difficult to find only correct dependencies)
 // Deoptimize all compiled code that depends on this class.
 //
 // If the can_redefine_classes capability is obtained in the onload
@@ -1769,12 +1762,9 @@ void VM_EnhancedRedefineClasses::flush_dependent_code(InstanceKlass* k_h, TRAPS)
   }
 }
 
-/**
-  Compare _old_methods and _new_methods arrays and store the result into
-	_matching_old_methods, _matching_new_methods, _added_methods, _deleted_methods
-
-  Setup _old_methods and _new_methods before the call - it should be called for one class only!
-*/
+//  Compare _old_methods and _new_methods arrays and store the result into
+//  _matching_old_methods, _matching_new_methods, _added_methods, _deleted_methods
+//  Setup _old_methods and _new_methods before the call - it should be called for one class only!
 void VM_EnhancedRedefineClasses::compute_added_deleted_matching_methods() {
   Method* old_method;
   Method* new_method;
@@ -1852,10 +1842,6 @@ void VM_EnhancedRedefineClasses::redefine_single_class(InstanceKlass* new_class_
 
   HandleMark hm(THREAD);   // make sure handles from this call are freed
 
-  if (log_is_enabled(Info, redefine, class, timer)) {
-    _timer_rsc_phase1.start();
-  }
-
   InstanceKlass* new_class = new_class_oop;
   InstanceKlass* the_class = InstanceKlass::cast(new_class_oop->old_version());
   assert(the_class != NULL, "must have old version");
@@ -1898,7 +1884,6 @@ void VM_EnhancedRedefineClasses::redefine_single_class(InstanceKlass* new_class_
   }
   */
 
-
   {
     ResourceMark rm(THREAD);
     // increment the classRedefinedCount field in the_class and in any
@@ -1911,7 +1896,6 @@ void VM_EnhancedRedefineClasses::redefine_single_class(InstanceKlass* new_class_
                              new_class->external_name(),
                              java_lang_Class::classRedefinedCount(new_class->java_mirror()));
   }
-  _timer_rsc_phase2.stop();
 } // end redefine_single_class()
 
 
@@ -1938,9 +1922,7 @@ void VM_EnhancedRedefineClasses::check_class(InstanceKlass* ik, TRAPS) {
   }
 }
 
-/**
- * Logging of all methods (old, new, changed, ...)
- */
+// Logging of all methods (old, new, changed, ...)
 void VM_EnhancedRedefineClasses::dump_methods() {
   int j;
   log_trace(redefine, class, dump)("_old_methods --");
@@ -2000,9 +1982,7 @@ void VM_EnhancedRedefineClasses::dump_methods() {
   }
 }
 
-/**
-  Helper class to traverse all loaded classes and figure out if the class is affected by redefinition.
-*/
+// Helper class to traverse all loaded classes and figure out if the class is affected by redefinition.
 class AffectedKlassClosure : public KlassClosure {
  private:
    GrowableArray<Klass*>* _affected_klasses;
@@ -2050,10 +2030,8 @@ class AffectedKlassClosure : public KlassClosure {
   }
 };
 
-/**
-  Find all affected classes by current redefinition (either because of redefine, class hierarchy or interface change).
-  Affected classes are stored in _affected_klasses and parent classes always precedes child class.
-*/
+// Find all affected classes by current redefinition (either because of redefine, class hierarchy or interface change).
+// Affected classes are stored in _affected_klasses and parent classes always precedes child class.
 jvmtiError VM_EnhancedRedefineClasses::find_sorted_affected_classes(TRAPS) {
   for (int i = 0; i < _class_count; i++) {
     InstanceKlass* klass_handle = get_ik(_class_defs[i].klass);
@@ -2084,9 +2062,7 @@ jvmtiError VM_EnhancedRedefineClasses::find_sorted_affected_classes(TRAPS) {
   return JVMTI_ERROR_NONE;
 }
 
-/**
-  Pairs of class dependencies (for topological sort)
-*/
+// Pairs of class dependencies (for topological sort)
 struct KlassPair {
   const Klass* _left;
   const Klass* _right;
@@ -2099,14 +2075,11 @@ static bool match_second(void* value, KlassPair elem) {
   return elem._right == value;
 }
 
-/**
- For each class to be redefined parse the bytecode and figure out the superclass and all interfaces.
- First newly introduced classes (_class_defs) are scanned and then affected classed (_affected_klasses).
- Affected flag is cleared (clear_redefinition_flag(Klass::MarkedAsAffected))
- For each dependency create a KlassPair instance. Finnaly, affected classes (_affected_klasses) are sorted according to pairs.
-
- TODO - the class file is potentionally parsed multiple times - introduce a cache?
-*/
+// For each class to be redefined parse the bytecode and figure out the superclass and all interfaces.
+// First newly introduced classes (_class_defs) are scanned and then affected classed (_affected_klasses).
+// Affected flag is cleared (clear_redefinition_flag(Klass::MarkedAsAffected))
+// For each dependency create a KlassPair instance. Finally, affected classes (_affected_klasses) are sorted according to pairs.
+// TODO - the class file is potentially parsed multiple times - introduce a cache?
 jvmtiError VM_EnhancedRedefineClasses::do_topological_class_sorting(TRAPS) {
   ResourceMark mark(THREAD);
 
